@@ -76,6 +76,9 @@ class LinearSolverType(Enum):
 
 class StepControlType(Enum):
     Exact = auto()
+    Fixed = auto()
+    Optimizing = auto()
+    BoxReduced = auto()
     ResiduumRatio = auto()
     DistanceRatio = auto()
 
@@ -169,7 +172,7 @@ class Params:
     validate_input: bool = True
 
     iteration_limit: Optional[int] = None
-    time_limit: float = np.inf
+    time_limit: float = float(np.inf)
     display_interval: float = 0.1
 
     # lower bound on objective function value,
