@@ -1,18 +1,25 @@
 import time
 from enum import Enum, auto
-from typing import Optional
+from typing import Optional, cast
 
 import numpy as np
 
-from pygradflow.cons_problem import ConstrainedProblem
 from pygradflow.display import Format, problem_display
+from pygradflow.eval import Evaluator, SimpleEvaluator, ValidatingEvaluator
 from pygradflow.iterate import Iterate
 from pygradflow.log import logger
 from pygradflow.newton import newton_method
 from pygradflow.params import Params
 from pygradflow.penalty import penalty_strategy
 from pygradflow.problem import Problem
-from pygradflow.step.step_control import StepController, StepResult, step_controller
+from pygradflow.scale import Scaling
+from pygradflow.step.step_control import (
+    StepController,
+    StepControlResult,
+    step_controller,
+)
+from pygradflow.timer import Timer
+from pygradflow.transform import Transformation
 
 
 class SolverStatus(Enum):
@@ -155,13 +162,17 @@ class Solver:
         \\begin{align}
             \\nabla_x f(x) + y^{T} J_c(x) + d = 0,
         \\end{align}
+
     feasibility
-    .. math::
+
+     .. math::
         \\begin{align}
             \\quad & l \\leq c(x) \\leq u \\\\
             \\quad & l^x \\leq x \\leq u^x \\\\
         \\end{align}
+
     and complementarity
+
      .. math::
         \\begin{align}
             y_i
@@ -177,10 +188,16 @@ class Solver:
                 = 0 & \\text{ otherwise }
             \\end{cases}
         \\end{align}
+
     conditions.
     """
 
-    def __init__(self, problem: Problem, params: Params = Params()) -> None:
+    def __init__(
+        self,
+        problem: Problem,
+        params: Params = Params(),
+        scaling: Optional[Scaling] = None,
+    ) -> None:
         """
         Creates a new solver
 
@@ -193,24 +210,28 @@ class Solver:
         """
         self.orig_problem = problem
         self.params = params
+        self.scaling = scaling
 
-        self.problem = ConstrainedProblem(problem)
+        self.transform = Transformation(problem, params, scaling)
+
+        self.problem = self.transform.trans_problem
 
         if params.validate_input:
-            from .eval import ValidatingEvaluator
-
-            self.evaluator = ValidatingEvaluator(self.problem, params)
+            self.evaluator: Evaluator = ValidatingEvaluator(self.problem, params)
         else:
-            from .eval import SimpleEvaluator
-
             self.evaluator = SimpleEvaluator(self.problem, params)
 
         self.penalty = penalty_strategy(self.problem, params)
         self.rho = -1.0
 
     def compute_step(
-        self, controller: StepController, iterate: Iterate, dt: float
-    ) -> StepResult:
+        self,
+        controller: StepController,
+        iterate: Iterate,
+        dt: float,
+        display: bool,
+        timer: Timer,
+    ) -> StepControlResult:
         problem = self.problem
         params = self.params
         assert self.rho != -1.0
@@ -224,7 +245,9 @@ class Solver:
                 yield next_step
                 curr_iterate = next_step.iterate
 
-        return controller.compute_step(iterate, self.rho, dt, next_steps())
+        return controller.compute_step(
+            iterate, self.rho, dt, next_steps(), display, timer
+        )
 
     def _deriv_check(self, x: np.ndarray, y: np.ndarray) -> None:
         from pygradflow.deriv_check import deriv_check
@@ -254,6 +277,8 @@ class Solver:
                 params,
             )
 
+        logger.info("Finished derivative check")
+
     def print_result(
         self,
         total_time: float,
@@ -281,10 +306,45 @@ class Solver:
         logger.info("%20s: %45e", "Aug Lag violation", iterate.aug_lag_violation(rho))
         logger.info("%20s: %45e", "Aug Lag dual", iterate.aug_lag_dual())
 
-        logger.info("%20s: %45e", "Bound violation", iterate.bound_violation)
         logger.info("%20s: %45e", "Constraint violation", iterate.cons_violation)
         logger.info("%20s: %45e", "Dual violation", iterate.stat_res)
 
+    def _create_initial_iterate(
+        self, x0: Optional[np.ndarray], y0: Optional[np.ndarray]
+    ):
+        params = self.params
+        dtype = params.dtype
+        orig_problem = self.orig_problem
+        problem = self.problem
+
+        orig_lb = orig_problem.var_lb
+        orig_ub = orig_problem.var_ub
+
+        if x0 is None:
+            orig_n = orig_problem.num_vars
+            x_init = np.zeros((orig_n,), dtype=dtype)
+            np.clip(x_init, orig_lb, orig_ub, out=x_init)
+        else:
+            if (x0 > orig_ub).any() or (x0 < orig_lb).any():
+                logger.warning("Initial point violates variable bounds")
+                x0 = np.clip(x0, orig_lb, orig_ub)
+
+            x_init = cast(np.ndarray, x0)
+
+        if y0 is None:
+            orig_m = orig_problem.num_cons
+            y_init = np.zeros((orig_m,), dtype=dtype)
+        else:
+            y_init = cast(np.ndarray, y0)
+
+        transform = self.transform
+        (x_init, y_init) = transform.transform_sol(x_init, y_init)
+
+        x = x_init.astype(dtype)
+        y = y_init.astype(dtype)
+
+        return Iterate(problem, params, x, y, self.evaluator)
+
     def solve(
         self, x0: Optional[np.ndarray] = None, y0: Optional[np.ndarray] = None
     ) -> SolverResult:
@@ -306,31 +366,13 @@ class Solver:
         """
         problem = self.problem
         params = self.params
-        dtype = params.dtype
 
         display = problem_display(problem, params)
 
         n = problem.num_vars
         m = problem.num_cons
 
-        orig_problem = self.orig_problem
-
-        if x0 is None:
-            orig_n = orig_problem.num_vars
-            x0 = np.zeros((orig_n,), dtype=dtype)
-            x0 = np.clip(x0, orig_problem.var_lb, orig_problem.var_ub)
-
-        if y0 is None:
-            orig_m = orig_problem.num_cons
-            y0 = np.zeros((orig_m,), dtype=dtype)
-
-        (x0, y0) = problem.transform_sol(x0, y0)
-
-        x = x0.astype(dtype)
-        y = y0.astype(dtype)
-
-        assert (n,) == x.shape
-        assert (m,) == y.shape
+        iterate = self._create_initial_iterate(x0, y0)
 
         logger.info("Solving problem with {0} variables, {1} constraints".format(n, m))
 
@@ -338,9 +380,8 @@ class Solver:
 
         controller = step_controller(problem, params)
 
-        self._deriv_check(x, y)
+        self._deriv_check(iterate.x, iterate.y)
 
-        iterate = Iterate(problem, params, x, y, self.evaluator)
         self.rho = self.penalty.initial(iterate)
 
         logger.debug("Initial Aug Lag: %.10e", iterate.aug_lag(self.rho))
@@ -358,6 +399,11 @@ class Solver:
         accepted_steps = 0
         iteration = 0
 
+        last_active_set = None
+        last_display_iteration = -1
+
+        timer = Timer(params.time_limit)
+
         while True:
             if line_diff == header_interval:
                 line_diff = 0
@@ -380,7 +426,12 @@ class Solver:
                 status = SolverStatus.Unbounded
                 break
 
-            step_result = self.compute_step(controller, iterate, 1.0 / lamb)
+            curr_time = time.time()
+            display_iterate = curr_time - last_time >= params.display_interval
+
+            step_result = self.compute_step(
+                controller, iterate, 1.0 / lamb, display_iterate, timer
+            )
 
             x = iterate.x
             y = iterate.y
@@ -397,21 +448,28 @@ class Solver:
             primal_step_norm = np.linalg.norm(next_iterate.x - iterate.x)
             dual_step_norm = np.linalg.norm(next_iterate.y - iterate.y)
 
-            curr_time = time.time()
-
-            if curr_time - start_time >= params.time_limit:
+            if timer.reached_time_limit():
                 logger.debug("Reached time limit")
                 status = SolverStatus.TimeLimit
                 break
 
-            if curr_time - last_time >= params.display_interval:
+            if display_iterate:
                 last_time = curr_time
                 line_diff += 1
 
                 state = dict()
                 state["iterate"] = iterate
 
+                def compute_last_active_set():
+                    if last_display_iteration + 1 == iteration:
+                        return last_active_set
+                    return None
+
+                state["last_active_set"] = compute_last_active_set
+                state["curr_active_set"] = lambda: step_result.active_set
+
                 state["aug_lag"] = lambda: iterate.aug_lag(self.rho)
+                state["obj"] = lambda: iterate.obj()
                 state["iter"] = lambda: iteration + 1
                 state["primal_step_norm"] = lambda: primal_step_norm
                 state["dual_step_norm"] = lambda: dual_step_norm
@@ -420,6 +478,7 @@ class Solver:
                 state["rcond"] = lambda: step_result.rcond
 
                 logger.info(display.row(state))
+                last_display_iteration = iteration
 
             if accept:
                 # Accept
@@ -444,6 +503,8 @@ class Solver:
                     break
 
             iteration += 1
+            last_active_set = step_result.active_set
+            # last_active_set = iterate.active_set
 
             if (params.iteration_limit is not None) and (
                 iteration >= params.iteration_limit
@@ -452,8 +513,7 @@ class Solver:
                 logger.debug("Iteration limit reached")
                 break
 
-        curr_time = time.time()
-        total_time = curr_time - start_time
+        total_time = timer.elapsed()
 
         direct_dist = iterate.dist(initial_iterate)
 
@@ -475,9 +535,11 @@ class Solver:
 
         x = iterate.x
         y = iterate.y
-        d = iterate.bound_duals
+        d = iterate.bounds_dual
 
-        (x, y, d) = problem.restore_sol(x, y, d)
+        transform = self.transform
+
+        (x, y, d) = transform.restore_sol(x, y, d)
 
         return SolverResult(
             x,
